@@ -1,5 +1,6 @@
 import fcntl
 import os
+import signal
 import struct
 import subprocess
 import sys
@@ -7,6 +8,7 @@ import termios
 import tty
 from dataclasses import dataclass, field
 from typing import List, Optional, TextIO, Tuple, Union
+from shared_utils import hard_exit
 
 tees = []
 
@@ -23,6 +25,7 @@ class Tee:
         tees.append(self)  # Do not let this get GC'd
         self.orig_stdout = _dup(sys.stdout)
         self.orig_stderr = _dup(sys.stderr)
+        hard_exit.register_atexit(self.close)
 
     def to(self, stdout, stderr):
         if not isinstance(stdout, list):
@@ -37,8 +40,16 @@ class Tee:
         self.stderr_pipe_proc = _tee(self.orig_stderr, stderr, stdout=self.orig_stdout)
         self.stderr = stderr
 
+        self._drain(stdout_pipe_proc=old_stdout_pipe_proc, stderr_pipe_proc=old_stderr_pipe_proc)
         self.resume()
 
+        return self
+
+    def close(self):
+        self.pause()
+        self._drain(self.stdout_pipe_proc, self.stderr_pipe_proc)
+
+    def _drain(self, stdout_pipe_proc, stderr_pipe_proc):
         # One sharp edge is that if you've spawned a subprocess with
         # the redirected stdout/stderr, the tee processes will not
         # die. In that case maybe we should set a timeout, or just
@@ -46,12 +57,13 @@ class Tee:
         if old_stdout_pipe_proc is not None:
             pipe, proc = old_stdout_pipe_proc
             pipe.close()
+            os.kill(signal.SIGINT, proc.pid)
             proc.wait()
         if old_stderr_pipe_proc is not None:
             pipe, proc = old_stderr_pipe_proc
             pipe.close()
+            os.kill(signal.SIGINT, proc.pid)
             proc.wait()
-        return self
 
     def resume(self):
         os.dup2(self.stdout_pipe_proc[0].fileno(), sys.stdout.fileno())
